@@ -9,7 +9,7 @@ from django.conf import settings
 from django.core.serializers.json import DjangoJSONEncoder
 from django.utils import dateparse, html, timezone
 
-from .common import all_subclasses, get_optimal_decimal_places
+from .common import all_subclasses, debug_log, get_optimal_decimal_places
 
 ASC, DSC = "asc", "dsc"
 
@@ -78,7 +78,9 @@ class BaseType(metaclass=TypeMeta):
         try:
             return cls._parse(value, choices), None
         except Exception as e:
-            return None, str(e) if str(e) else repr(e)
+            debug_log("Error parsing filter value", e)
+            msg = str(e) if str(e) else repr(e)
+            return None, msg[0].upper() + msg[1:]
 
     @classmethod
     def parse_lookup(cls, lookup, value, choices):
@@ -90,6 +92,11 @@ class BaseType(metaclass=TypeMeta):
             type_ = TYPES[type_name]
             return type_.parse(value, choices)
 
+    @classmethod
+    def format_lookup(cls, lookup, value, choices):
+        pretty_name, type_name = cls.lookups[lookup]
+        return TYPES[type_name]._get_formatter(choices)(value)
+
     @staticmethod
     def get_format_hints(name, data):
         return {}
@@ -166,8 +173,12 @@ class RegexType(BaseType):
         # this is dirty
         # we need to check if the regex is going to cause a db exception
         # and not kill any in progress transaction as we check
-        with atomic():
-            list(ContentType.objects.filter(model__regex=value))
+        try:
+            with atomic():
+                list(ContentType.objects.filter(model__regex=value))
+        except Exception as e:
+            debug_log("Error parsing filter value", e)
+            raise Exception("Invalid regex")
         return value
 
 
@@ -192,7 +203,8 @@ class DurationType(BaseType):
             value += ":0"
 
         res = dateparse.parse_duration(value)
-        assert res is not None, "Duration value should be 'DD HH:MM:SS'"
+        if res is None:
+            raise Exception("Duration value should be 'DD HH:MM:SS'")
         return res
 
     @staticmethod
@@ -202,9 +214,7 @@ class DurationType(BaseType):
 
 
 class DateTypeMixin:
-    _clause_str = r"(\w{3,})([+-=])(\d+) *"
-    _clause = re.compile(_clause_str)
-    _clauses = re.compile(fr"^({_clause_str})+$")
+    _clause = re.compile(r"(\w{3,})([-+=])(\d+) *")
 
     _clause_types = {
         "yea": "year",
@@ -257,41 +267,54 @@ class DateTypeMixin:
                     dateutil.parser.parse(value, dayfirst=False, yearfirst=True),
                     dateutil.parser.parse(value, dayfirst=True, yearfirst=True),
                 }
-                assert len(res) == 1, "Ambiguous value"
+                if len(res) != 1:
+                    raise Exception("Ambiguous value")
                 res = res.pop()
             except dateutil.parser.ParserError:
                 # failing that must be relative delta stuff
                 res = timezone.now()
-                assert cls._clauses.match(value), "Unrecognized value"
 
-                for match in cls._clause.finditer(value):
+                for clause_str in value.split():
+                    match = cls._clause.fullmatch(clause_str)
+                    if not match:
+                        raise Exception(f"Unrecognized clause '{clause_str}'")
+
                     field, op, val = match.groups()
                     val = int(val)
+                    human_op = {
+                        "+": "add",
+                        "-": "subtract",
+                        "=": "set",
+                    }[op]
 
                     for prefix, arg in cls._clause_types.items():
                         if field.startswith(prefix):
                             break
                     else:
-                        assert False, f"Unrecognized field {field}"
+                        raise Exception(f"Unrecognized field '{field}'")
 
                     if isinstance(arg, str):
                         if op == "+":
                             kwargs = {f"{arg}s": val}
                         elif op == "-":
                             kwargs = {f"{arg}s": -val}
-                        else:
+                        else:  # op == "="
                             if arg in ["year", "month", "day"]:
-                                assert val > 0, f"Can't set {field} to {val}"
+                                if val <= 0:
+                                    raise Exception(
+                                        f"Can't {human_op} '{field}' to '{val}'"
+                                    )
                             kwargs = {arg: val}
                     else:
-                        assert val > 0, f"Can't set {field} to {val}"
-                        if op == "+":
-                            kwargs = {"weekday": arg(val)}
-                        elif op == "-":
-                            kwargs = {"weekday": arg(-val)}
+                        if op == "=":
+                            raise Exception(f"'{op}' not supported for '{field}'")
                         else:
-                            # =
-                            assert False, f"{op} not supported for {field}"
+                            if val <= 0:
+                                raise Exception(f"Can't {human_op} '{val}' '{field}'s")
+                            if op == "+":
+                                kwargs = {"weekday": arg(val)}
+                            else:  # op == "-"
+                                kwargs = {"weekday": arg(-val)}
 
                     res += relativedelta.relativedelta(**kwargs)
                 return res
@@ -478,7 +501,7 @@ class IsNullType(ChoiceTypeMixin, BaseType):
 
 
 class ArrayTypeMixin:
-    default_value = None
+    default_value = "[]"
 
     @classmethod
     def _get_formatter(cls, choices):  # pragma: postgres
@@ -508,7 +531,7 @@ class ArrayTypeMixin:
         value = _json_loads(value)
         if not isinstance(value, list):
             raise ValueError("Expected a list")
-        return [cls.element_type._parse(v, choices)[0] for v in value]
+        return [cls.element_type._parse(v, choices) for v in value]
 
 
 class StringArrayType(ArrayTypeMixin, BaseType):
