@@ -40,10 +40,7 @@ def _get_query_data(bound_query):
         ],
         "filterErrors": [filter_.err_message for filter_ in bound_query.filters],
         "parsedFilterValues": [
-            filter_.orm_bound_field.get_formatter()(filter_.parsed)
-            if filter_.is_valid
-            else None
-            for filter_ in bound_query.filters
+            filter_.formatted_value() for filter_ in bound_query.filters
         ],
         "fields": [
             {
@@ -63,8 +60,15 @@ def _get_model_fields(model_name, orm_models):
     orm_model = orm_models[model_name]
 
     def sort_model_fields(fields):
-        front = {"id": 1, OPEN_IN_ADMIN: 2}
-        return sorted(fields, key=lambda f: (front.get(f, sys.maxsize), f))
+        front = {orm_model.pk: 1, OPEN_IN_ADMIN: 2}
+        sorted_fields = sorted(
+            fields.items(),
+            key=lambda name_field: (
+                front.get(name_field[0], sys.maxsize),
+                name_field[1]["prettyName"],
+            ),
+        )
+        return [name for name, field in sorted_fields]
 
     all_fields = {
         name: {
@@ -372,7 +376,7 @@ def _get_from_js_dev_server(request, method=None):  # pragma: no cover
         method = request.META["REQUEST_METHOD"].lower()
 
     upstream_url = f"http://127.0.0.1:3000{request.path}"
-    print(f"Proxy request: {method} {upstream_url}")
+    sys.stdout.write(f"Proxy request: {method} {upstream_url}\n")
     return getattr(requests, method)(upstream_url, stream=True)
 
 
