@@ -43,6 +43,7 @@ def products(db):
             producer=producer,
             fake="phoney",
             boat=2.2,
+            date="2001-01-01",
         )
     )
 
@@ -63,6 +64,7 @@ def products(db):
             size_unit="g",
             producer=producer,
             boat=2.4,
+            date="2002-02-02",
         )
     )
 
@@ -155,7 +157,7 @@ def test_get_file_link(get_product_flat):
     models.Product.objects.create(name="a", producer=producer)
     models.Product.objects.create(name="b", producer=producer, image="bob.jpg")
     data = get_product_flat(1, "name,image", [])
-    sortedAssert(data, [["a", None], ["b", '<a href="/media/bob.jpg">bob.jpg</a>']])
+    sortedAssert(data, [["a", ""], ["b", '<a href="/media/bob.jpg">bob.jpg</a>']])
 
 
 def test_bad_storage(monkeypatch, req):
@@ -184,7 +186,7 @@ def test_bad_storage(monkeypatch, req):
     models.Product.objects.create(name="a", producer=producer)
     models.Product.objects.create(name="b", producer=producer, image="bob.jpg")
     data = get_product_flat("name,image", [])
-    sortedAssert(data, [["a", None], ["b", "assert False"]])
+    sortedAssert(data, [["a", ""], ["b", "assert False"]])
 
 
 @pytest.mark.usefixtures("products")
@@ -247,10 +249,10 @@ def test_get_annotated_field_down_tree(products, get_product_flat, mocker):
     )
     data = get_product_flat(
         1,
-        "producer__address__andrew+1,size-2",
+        "producer__address__andrew+2,size-1",
         [("producer__address__andrew__not_equals", "bad")],
     )
-    assert data == [["good", 1]]
+    assert data == [[None, 2.0], ["good", 1]]
     assert len(mock.call_args_list) == 2
 
 
@@ -324,6 +326,26 @@ def test_filter_year_bounds(get_product_flat):
     assert data == [[1], [2]]  # this errors on the filter and returns all values
 
 
+@pytest.mark.usefixtures("products")
+def test_filter_year_not_equals(get_product_flat):
+    # https://github.com/tolomea/django-data-browser/issues/27
+    data = get_product_flat(1, "name+1", [("date__year__equals", 2001)])
+    assert data == [["a"]]
+
+    data = get_product_flat(1, "name+1", [("date__year__not_equals", 2001)])
+    assert data == [["b"], ["c"]]
+
+
+@pytest.mark.usefixtures("products")
+def test_filter_year_is_null(get_product_flat):
+    data = get_product_flat(1, "name+1", [("date__year__is_null", "NotNull")])
+    assert data == [["a"], ["c"]]
+
+    data = get_product_flat(1, "name+1", [("date__year__is_null", "IsNull")])
+    assert data == [["b"]]
+
+
+@pytest.mark.usefixtures("products")
 @pytest.mark.usefixtures("products")
 def test_get_is_null_function(get_product_flat):
     data = get_product_flat(1, "name+0,producer__address__city__is_null", [])
@@ -468,6 +490,11 @@ def test_get_results_null_filter(get_product_flat):
     data = get_product_flat(1, "name", [("onsale__is_null", "NotNull")])
     sortedAssert(data, [])
 
+    data = get_product_flat(1, "name", [("created_time__is_null", "IsNull")])
+    sortedAssert(data, [])
+    data = get_product_flat(1, "name", [("created_time__is_null", "NotNull")])
+    sortedAssert(data, [["a"], ["b"], ["c"]])
+
 
 @pytest.mark.usefixtures("products")
 def test_get_results_boolean_filter(get_product_flat):
@@ -730,16 +757,17 @@ class TestPermissions:
 
         assert "core.NotInAdmin" not in orm_models
         assert orm_models["core.InAdmin"] == OrmModel(
-            fields=KEYS("admin", "id", "name"), admin=ANY(BaseModelAdmin)
+            fields=KEYS("admin", "id", "name"), admin=ANY(BaseModelAdmin), pk="id"
         )
         assert orm_models["core.InlineAdmin"] == OrmModel(
-            fields=KEYS("id", "name", "in_admin"), admin=ANY(BaseModelAdmin)
+            fields=KEYS("id", "name", "in_admin"), admin=ANY(BaseModelAdmin), pk="id"
         )
         assert orm_models["core.Normal"] == OrmModel(
             fields=KEYS(
                 "admin", "id", "name", "in_admin", "inline_admin", "not_in_admin"
             ),
             admin=ANY(BaseModelAdmin),
+            pk="id",
         )
 
     @pytest.mark.django_db
@@ -754,6 +782,7 @@ class TestPermissions:
                 "admin", "id", "name", "in_admin", "inline_admin", "not_in_admin"
             ),
             admin=ANY(BaseModelAdmin),
+            pk="id",
         )
 
     @pytest.mark.django_db
@@ -768,6 +797,7 @@ class TestPermissions:
                 "admin", "id", "name", "in_admin", "inline_admin", "not_in_admin"
             ),
             admin=ANY(BaseModelAdmin),
+            pk="id",
         )
 
     @pytest.mark.django_db
@@ -776,7 +806,7 @@ class TestPermissions:
 
         assert "core.NotInAdmin" not in orm_models
         assert orm_models["core.InAdmin"] == OrmModel(
-            fields=KEYS("admin", "id", "name"), admin=ANY(BaseModelAdmin)
+            fields=KEYS("admin", "id", "name"), admin=ANY(BaseModelAdmin), pk="id"
         )
         assert "core.InlineAdmin" not in orm_models
         assert orm_models["core.Normal"] == OrmModel(
@@ -784,6 +814,7 @@ class TestPermissions:
                 "admin", "id", "name", "in_admin", "inline_admin", "not_in_admin"
             ),
             admin=ANY(BaseModelAdmin),
+            pk="id",
         )
 
 
